import {
  ClassType,
  PropertyType,
} from '@loafer/core/constants/types';

import AnnotatedElement from './AnnotatedElement';
import Constructor from './Constructor';
import Property from './Property';
import Method from './Method';


export class Class extends AnnotatedElement {
  private _type: ClassType;
  private _name: PropertyType;
  private _constructor: Constructor;
  private _properties: Map<PropertyType, Property>;
  private _methodes: Map<PropertyType, Method>;

  public constructor(type: ClassType, parameterTypes: ClassType[]) {
    super();
    this._type = type;
    this._name = this._type.constructor.name;
    this._constructor = new Constructor(parameterTypes);
    this._properties = new Map();
    this._methodes = new Map();
  }

  public get Type(): ClassType {
    return this._type;
  }

  public get Name(): PropertyType {
    return this._name;
  }

  public get Constructor(): Constructor {
    return this._constructor;
  }

  public addProperty(propertyKey: PropertyType, type: ClassType): Property {
    if (this._properties.has(propertyKey)) {
      throw new Error(`Property[${propertyKey}:${type.constructor.name}] on Class[${this._name}] is exist already`);
    }
    let proerty = new Property(type, propertyKey);
    this._properties.set(propertyKey, proerty);
    return proerty;
  }

  public getProperty(propertyKey: PropertyType): Property {
    return this._properties.get(propertyKey);
  }

  public hasProperty(propertyKey: PropertyType): boolean {
    return this._properties.has(propertyKey);
  }

  public addMethod(propertyKey: PropertyType, parameterTypes: ClassType[], returnType: ClassType): Method {
    if (this._methodes.has(propertyKey)) {
      throw new Error(`Method[${propertyKey}:${returnType.constructor.name}] on Class[${this._name}] is exist already`);
    }
    let method = new Method(propertyKey, parameterTypes, returnType);
    this._methodes.set(propertyKey, method);
    return method;
  }

  public getMethod(propertyKey: PropertyType): Method {
    return this._methodes.get(propertyKey);
  }

  public hasMethod(propertyKey: PropertyType): boolean {
    return this._methodes.has(propertyKey);
  }
}

export default Class;
